@@ -28,7 +28,6 @@
     webpmux -set xmp image_metadata.xmp in.webp -o out_xmp_container.webp
 
   Extract relevant data from WebP container file:
-    webpmux -get frgm n in.webp -o out_fragment.webp
     webpmux -get frame n in.webp -o out_frame.webp
     webpmux -get icc in.webp -o image_profile.icc
     webpmux -get exif in.webp -o image_metadata.exif
@@ -39,6 +38,11 @@
     webpmux -strip exif in.webp -o out.webp
     webpmux -strip xmp in.webp -o out.webp
 
+  Change duration of frame intervals:
+    webpmux -duration 150 in.webp -o out.webp
+    webpmux -duration 33,2 in.webp -o out.webp
+    webpmux -duration 200,10,0 -duration 150,6,50 in.webp -o out.webp
+
   Misc:
     webpmux -info in.webp
     webpmux [ -h | -help ]
@@ -55,7 +59,8 @@
 #include <string.h>
 #include "webp/decode.h"
 #include "webp/mux.h"
-#include "./example_util.h"
+#include "../examples/example_util.h"
+#include "../imageio/imageio_util.h"
 
 //------------------------------------------------------------------------------
 // Config object to parse command-line arguments.
@@ -66,7 +71,8 @@ typedef enum {
   ACTION_SET,
   ACTION_STRIP,
   ACTION_INFO,
-  ACTION_HELP
+  ACTION_HELP,
+  ACTION_DURATION
 } ActionType;
 
 typedef enum {
@@ -88,17 +94,17 @@ typedef enum {
   FEATURE_XMP,
   FEATURE_ICCP,
   FEATURE_ANMF,
-  FEATURE_FRGM,
+  FEATURE_DURATION,
   LAST_FEATURE
 } FeatureType;
 
 static const char* const kFourccList[LAST_FEATURE] = {
-  NULL, "EXIF", "XMP ", "ICCP", "ANMF", "FRGM"
+  NULL, "EXIF", "XMP ", "ICCP", "ANMF"
 };
 
 static const char* const kDescriptions[LAST_FEATURE] = {
   NULL, "EXIF metadata", "XMP metadata", "ICC profile",
-  "Animation frame", "Image fragment"
+  "Animation frame"
 };
 
 typedef struct {
@@ -182,7 +188,6 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
   printf("Canvas size: %d x %d\n", width, height);
 
   err = WebPMuxGetFeatures(mux, &flag);
-  if (flag & FRAGMENTS_FLAG) err = WEBP_MUX_INVALID_ARGUMENT;
   RETURN_IF_ERROR("Failed to retrieve features\n");
 
   if (flag == 0) {
@@ -193,26 +198,22 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
   // Print the features present.
   printf("Features present:");
   if (flag & ANIMATION_FLAG) printf(" animation");
-  if (flag & FRAGMENTS_FLAG) printf(" image fragments");
   if (flag & ICCP_FLAG)      printf(" ICC profile");
   if (flag & EXIF_FLAG)      printf(" EXIF metadata");
   if (flag & XMP_FLAG)       printf(" XMP metadata");
   if (flag & ALPHA_FLAG)     printf(" transparency");
   printf("\n");
 
-  if ((flag & ANIMATION_FLAG) || (flag & FRAGMENTS_FLAG)) {
-    const int is_anim = !!(flag & ANIMATION_FLAG);
-    const WebPChunkId id = is_anim ? WEBP_CHUNK_ANMF : WEBP_CHUNK_FRGM;
-    const char* const type_str = is_anim ? "frame" : "fragment";
+  if (flag & ANIMATION_FLAG) {
+    const WebPChunkId id = WEBP_CHUNK_ANMF;
+    const char* const type_str = "frame";
     int nFrames;
 
-    if (is_anim) {
-      WebPMuxAnimParams params;
-      err = WebPMuxGetAnimationParams(mux, &params);
-      assert(err == WEBP_MUX_OK);
-      printf("Background color : 0x%.8X  Loop Count : %d\n",
-             params.bgcolor, params.loop_count);
-    }
+    WebPMuxAnimParams params;
+    err = WebPMuxGetAnimationParams(mux, &params);
+    assert(err == WEBP_MUX_OK);
+    printf("Background color : 0x%.8X  Loop Count : %d\n",
+           params.bgcolor, params.loop_count);
 
     err = WebPMuxNumChunks(mux, id, &nFrames);
     assert(err == WEBP_MUX_OK);
@@ -221,8 +222,8 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
     if (nFrames > 0) {
       int i;
       printf("No.: width height alpha x_offset y_offset ");
-      if (is_anim) printf("duration   dispose blend ");
-      printf("image_size\n");
+      printf("duration   dispose blend ");
+      printf("image_size  compression\n");
       for (i = 1; i <= nFrames; i++) {
         WebPMuxFrameInfo frame;
         err = WebPMuxGetFrame(mux, i, &frame);
@@ -235,7 +236,7 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
           printf("%3d: %5d %5d %5s %8d %8d ", i, features.width,
                  features.height, features.has_alpha ? "yes" : "no",
                  frame.x_offset, frame.y_offset);
-          if (is_anim) {
+          {
             const char* const dispose =
                 (frame.dispose_method == WEBP_MUX_DISPOSE_NONE) ? "none"
                                                                 : "background";
@@ -243,7 +244,10 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
                 (frame.blend_method == WEBP_MUX_BLEND) ? "yes" : "no";
             printf("%8d %10s %5s ", frame.duration, dispose, blend);
           }
-          printf("%10d\n", (int)frame.bitstream.size);
+          printf("%10d %11s\n", (int)frame.bitstream.size,
+                 (features.format == 1) ? "lossy" :
+                 (features.format == 2) ? "lossless" :
+                                          "undefined");
         }
         WebPDataClear(&frame.bitstream);
         RETURN_IF_ERROR3("Failed to retrieve %s#%d\n", type_str, i);
@@ -272,7 +276,7 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
     printf("Size of the XMP metadata: %d\n", (int)xmp.size);
   }
 
-  if ((flag & ALPHA_FLAG) && !(flag & (ANIMATION_FLAG | FRAGMENTS_FLAG))) {
+  if ((flag & ALPHA_FLAG) && !(flag & ANIMATION_FLAG)) {
     WebPMuxFrameInfo image;
     err = WebPMuxGetFrame(mux, 1, &image);
     if (err == WEBP_MUX_OK) {
@@ -288,6 +292,8 @@ static WebPMuxError DisplayInfo(const WebPMux* mux) {
 static void PrintHelp(void) {
   printf("Usage: webpmux -get GET_OPTIONS INPUT -o OUTPUT\n");
   printf("       webpmux -set SET_OPTIONS INPUT -o OUTPUT\n");
+  printf("       webpmux -duration DURATION_OPTIONS [-duration ...]\n");
+  printf("               INPUT -o OUTPUT\n");
   printf("       webpmux -strip STRIP_OPTIONS INPUT -o OUTPUT\n");
   printf("       webpmux -frame FRAME_OPTIONS [-frame...] [-loop LOOP_COUNT]"
          "\n");
@@ -314,6 +320,18 @@ static void PrintHelp(void) {
   printf("             'file.exif' contains the EXIF metadata to be set\n");
   printf("             'file.xmp' contains the XMP metadata to be set\n");
 
+  printf("\n");
+  printf("DURATION_OPTIONS:\n");
+  printf(" Set duration of selected frames:\n");
+  printf("   duration            set duration for each frames\n");
+  printf("   duration,frame      set duration of a particular frame\n");
+  printf("   duration,start,end  set duration of frames in the\n");
+  printf("                        interval [start,end])\n");
+  printf("   where: 'duration' is the duration in milliseconds\n");
+  printf("          'start' is the start frame index\n");
+  printf("          'end' is the inclusive end frame index\n");
+  printf("           The special 'end' value '0' means: last frame.\n");
+
   printf("\n");
   printf("STRIP_OPTIONS:\n");
   printf(" Strip color profile/metadata:\n");
@@ -365,7 +383,7 @@ static int ReadFileToWebPData(const char* const filename,
                               WebPData* const webp_data) {
   const uint8_t* data;
   size_t size;
-  if (!ExUtilReadFile(filename, &data, &size)) return 0;
+  if (!ImgIoUtilReadFile(filename, &data, &size)) return 0;
   webp_data->bytes = data;
   webp_data->size = size;
   return 1;
@@ -385,7 +403,7 @@ static int CreateMux(const char* const filename, WebPMux** mux) {
 static int WriteData(const char* filename, const WebPData* const webpdata) {
   int ok = 0;
   FILE* fout = strcmp(filename, "-") ? fopen(filename, "wb")
-                                     : ExUtilSetBinaryMode(stdout);
+                                     : ImgIoUtilSetBinaryMode(stdout);
   if (fout == NULL) {
     fprintf(stderr, "Error opening output WebP file %s!\n", filename);
     return 0;
@@ -414,6 +432,45 @@ static int WriteWebP(WebPMux* const mux, const char* filename) {
   return ok;
 }
 
+static WebPMux* DuplicateMuxHeader(const WebPMux* const mux) {
+  WebPMux* new_mux = WebPMuxNew();
+  WebPMuxAnimParams p;
+  WebPMuxError err;
+  int i;
+  int ok = 1;
+
+  if (new_mux == NULL) return NULL;
+
+  err = WebPMuxGetAnimationParams(mux, &p);
+  if (err == WEBP_MUX_OK) {
+    err = WebPMuxSetAnimationParams(new_mux, &p);
+    if (err != WEBP_MUX_OK) {
+      ERROR_GOTO2("Error (%s) handling animation params.\n",
+                  ErrorString(err), End);
+    }
+  } else {
+    /* it might not be an animation. Just keep moving. */
+  }
+
+  for (i = 1; i <= 3; ++i) {
+    WebPData metadata;
+    err = WebPMuxGetChunk(mux, kFourccList[i], &metadata);
+    if (err == WEBP_MUX_OK && metadata.size > 0) {
+      err = WebPMuxSetChunk(new_mux, kFourccList[i], &metadata, 1);
+      if (err != WEBP_MUX_OK) {
+        ERROR_GOTO1("Error transferring metadata in DuplicateMux().", End);
+      }
+    }
+  }
+
+ End:
+  if (!ok) {
+    WebPMuxDelete(new_mux);
+    new_mux = NULL;
+  }
+  return new_mux;
+}
+
 static int ParseFrameArgs(const char* args, WebPMuxFrameInfo* const info) {
   int dispose_method, dummy;
   char plus_minus, blend_method;
@@ -449,13 +506,6 @@ static int ParseFrameArgs(const char* args, WebPMuxFrameInfo* const info) {
   return 1;
 }
 
-static int ParseFragmentArgs(const char* args, WebPMuxFrameInfo* const info) {
-  const int ok =
-      (sscanf(args, "+%d+%d", &info->x_offset, &info->y_offset) == 2);
-  if (ok) WarnAboutOddOffset(info);
-  return ok;
-}
-
 static int ParseBgcolorArgs(const char* args, uint32_t* const bgcolor) {
   uint32_t a, r, g, b;
   if (sscanf(args, "%u,%u,%u,%u", &a, &r, &g, &b) != 4) return 0;
@@ -484,9 +534,9 @@ static void DeleteConfig(WebPMuxConfig* config) {
 static int ValidateCommandLine(int argc, const char* argv[],
                                int* num_feature_args) {
   int num_frame_args;
-  int num_frgm_args;
   int num_loop_args;
   int num_bgcolor_args;
+  int num_durations_args;
   int ok = 1;
 
   assert(num_feature_args != NULL);
@@ -511,9 +561,9 @@ static int ValidateCommandLine(int argc, const char* argv[],
 
   // Compound checks.
   num_frame_args = CountOccurrences(argv, argc, "-frame");
-  num_frgm_args = CountOccurrences(argv, argc, "-frgm");
   num_loop_args = CountOccurrences(argv, argc, "-loop");
   num_bgcolor_args = CountOccurrences(argv, argc, "-bgcolor");
+  num_durations_args = CountOccurrences(argv, argc, "-duration");
 
   if (num_loop_args > 1) {
     ERROR_GOTO1("ERROR: Multiple loop counts specified.\n", ErrValidate);
@@ -526,22 +576,20 @@ static int ValidateCommandLine(int argc, const char* argv[],
     ERROR_GOTO1("ERROR: Loop count and background color are relevant only in "
                 "case of animation.\n", ErrValidate);
   }
-  if (num_frame_args > 0 && num_frgm_args > 0) {
-    ERROR_GOTO1("ERROR: Only one of frames & fragments can be specified at a "
-                "time.\n", ErrValidate);
+  if (num_durations_args > 0 && num_frame_args != 0) {
+    ERROR_GOTO1("ERROR: Can not combine -duration and -frame commands.\n",
+                ErrValidate);
   }
 
   assert(ok == 1);
-  if (num_frame_args == 0 && num_frgm_args == 0) {
+  if (num_durations_args > 0) {
+    *num_feature_args = num_durations_args;
+  } else if (num_frame_args == 0) {
     // Single argument ('set' action for ICCP/EXIF/XMP, OR a 'get' action).
     *num_feature_args = 1;
   } else {
-    // Multiple arguments ('set' action for animation or fragmented image).
-    if (num_frame_args > 0) {
-      *num_feature_args = num_frame_args + num_loop_args + num_bgcolor_args;
-    } else {
-      *num_feature_args = num_frgm_args;
-    }
+    // Multiple arguments ('set' action for animation)
+    *num_feature_args = num_frame_args + num_loop_args + num_bgcolor_args;
   }
 
  ErrValidate:
@@ -583,6 +631,21 @@ static int ParseCommandLine(int argc, const char* argv[],
           ERROR_GOTO1("ERROR: Multiple actions specified.\n", ErrParse);
         }
         ++i;
+      } else if (!strcmp(argv[i], "-duration")) {
+        CHECK_NUM_ARGS_LESS(2, ErrParse);
+        if (ACTION_IS_NIL || config->action_type_ == ACTION_DURATION) {
+          config->action_type_ = ACTION_DURATION;
+        } else {
+          ERROR_GOTO1("ERROR: Multiple actions specified.\n", ErrParse);
+        }
+        if (FEATURETYPE_IS_NIL || feature->type_ == FEATURE_DURATION) {
+          feature->type_ = FEATURE_DURATION;
+        } else {
+          ERROR_GOTO1("ERROR: Multiple features specified.\n", ErrParse);
+        }
+        arg->params_ = argv[i + 1];
+        ++feature_arg_index;
+        i += 2;
       } else if (!strcmp(argv[i], "-get")) {
         if (ACTION_IS_NIL) {
           config->action_type_ = ACTION_GET;
@@ -694,8 +757,7 @@ static int ParseCommandLine(int argc, const char* argv[],
       } else if (!strcmp(argv[i], "frame") &&
                  (config->action_type_ == ACTION_GET)) {
         CHECK_NUM_ARGS_LESS(2, ErrParse);
-        feature->type_ = (!strcmp(argv[i], "frame")) ? FEATURE_ANMF :
-            FEATURE_FRGM;
+        feature->type_ = FEATURE_ANMF;
         arg->params_ = argv[i + 1];
         ++feature_arg_index;
         i += 2;
@@ -733,8 +795,7 @@ static int ValidateConfig(WebPMuxConfig* config) {
   if (config->input_ == NULL) {
     if (config->action_type_ != ACTION_SET) {
       ERROR_GOTO1("ERROR: No input file specified.\n", ErrValidate2);
-    } else if (feature->type_ != FEATURE_ANMF &&
-               feature->type_ != FEATURE_FRGM) {
+    } else if (feature->type_ != FEATURE_ANMF) {
       ERROR_GOTO1("ERROR: No input file specified.\n", ErrValidate2);
     }
   }
@@ -786,14 +847,13 @@ static int InitializeConfig(int argc, const char* argv[],
 //------------------------------------------------------------------------------
 // Processing.
 
-static int GetFrameFragment(const WebPMux* mux,
-                            const WebPMuxConfig* config, int is_frame) {
+static int GetFrame(const WebPMux* mux, const WebPMuxConfig* config) {
   WebPMuxError err = WEBP_MUX_OK;
   WebPMux* mux_single = NULL;
   int num = 0;
   int ok = 1;
   int parse_error = 0;
-  const WebPChunkId id = is_frame ? WEBP_CHUNK_ANMF : WEBP_CHUNK_FRGM;
+  const WebPChunkId id = WEBP_CHUNK_ANMF;
   WebPMuxFrameInfo info;
   WebPDataInit(&info.bitstream);
 
@@ -844,9 +904,7 @@ static int Process(const WebPMuxConfig* config) {
       if (!ok) goto Err2;
       switch (feature->type_) {
         case FEATURE_ANMF:
-        case FEATURE_FRGM:
-          ok = GetFrameFragment(mux, config,
-                                (feature->type_ == FEATURE_ANMF) ? 1 : 0);
+          ok = GetFrame(mux, config);
           break;
 
         case FEATURE_ICCP:
@@ -938,35 +996,6 @@ static int Process(const WebPMuxConfig* config) {
           break;
         }
 
-        case FEATURE_FRGM: {
-          int i;
-          mux = WebPMuxNew();
-          if (mux == NULL) {
-            ERROR_GOTO2("ERROR (%s): Could not allocate a mux object.\n",
-                        ErrorString(WEBP_MUX_MEMORY_ERROR), Err2);
-          }
-          for (i = 0; i < feature->arg_count_; ++i) {
-            WebPMuxFrameInfo frgm;
-            frgm.id = WEBP_CHUNK_FRGM;
-            ok = ReadFileToWebPData(feature->args_[i].filename_,
-                                    &frgm.bitstream);
-            if (!ok) goto Err2;
-            ok = ParseFragmentArgs(feature->args_[i].params_, &frgm);
-            if (!ok) {
-              WebPDataClear(&frgm.bitstream);
-              ERROR_GOTO1("ERROR: Could not parse fragment properties.\n",
-                          Err2);
-            }
-            err = WebPMuxPushFrame(mux, &frgm, 1);
-            WebPDataClear(&frgm.bitstream);
-            if (err != WEBP_MUX_OK) {
-              ERROR_GOTO3("ERROR (%s): Could not add a fragment at index %d.\n",
-                          ErrorString(err), i, Err2);
-            }
-          }
-          break;
-        }
-
         case FEATURE_ICCP:
         case FEATURE_EXIF:
         case FEATURE_XMP: {
@@ -990,6 +1019,89 @@ static int Process(const WebPMuxConfig* config) {
       ok = WriteWebP(mux, config->output_);
       break;
     }
+    case ACTION_DURATION: {
+      int num_frames;
+      ok = CreateMux(config->input_, &mux);
+      if (!ok) goto Err2;
+      err = WebPMuxNumChunks(mux, WEBP_CHUNK_ANMF, &num_frames);
+      ok = (err == WEBP_MUX_OK);
+      if (!ok) {
+        ERROR_GOTO1("ERROR: can not parse the number of frames.\n", Err2);
+      }
+      if (num_frames == 0) {
+        fprintf(stderr, "Doesn't look like the source is animated. "
+                        "Skipping duration setting.\n");
+        ok = WriteWebP(mux, config->output_);
+        if (!ok) goto Err2;
+      } else {
+        int i;
+        int* durations = NULL;
+        WebPMux* new_mux = DuplicateMuxHeader(mux);
+        if (new_mux == NULL) goto Err2;
+        durations = (int*)malloc((size_t)num_frames * sizeof(*durations));
+        if (durations == NULL) goto Err2;
+        for (i = 0; i < num_frames; ++i) durations[i] = -1;
+
+        // Parse intervals to process.
+        for (i = 0; i < feature->arg_count_; ++i) {
+          int k;
+          int args[3];
+          int duration, start, end;
+          const int nb_args = ExUtilGetInts(feature->args_[i].params_,
+                                            10, 3, args);
+          ok = (nb_args >= 1);
+          if (!ok) goto Err3;
+          duration = args[0];
+          if (duration < 0) {
+            ERROR_GOTO1("ERROR: duration must be strictly positive.\n", Err3);
+          }
+
+          if (nb_args == 1) {   // only duration is present -> use full interval
+            start = 1;
+            end = num_frames;
+          } else {
+            start = args[1];
+            if (start <= 0) {
+              start = 1;
+            } else if (start > num_frames) {
+              start = num_frames;
+            }
+            end = (nb_args >= 3) ? args[2] : start;
+            if (end == 0 || end > num_frames) end = num_frames;
+          }
+
+          for (k = start; k <= end; ++k) {
+            assert(k >= 1 && k <= num_frames);
+            durations[k - 1] = duration;
+          }
+        }
+
+        // Apply non-negative durations to their destination frames.
+        for (i = 1; i <= num_frames; ++i) {
+          WebPMuxFrameInfo frame;
+          err = WebPMuxGetFrame(mux, i, &frame);
+          if (err != WEBP_MUX_OK || frame.id != WEBP_CHUNK_ANMF) {
+            ERROR_GOTO2("ERROR: can not retrieve frame #%d.\n", i, Err3);
+          }
+          if (durations[i - 1] >= 0) frame.duration = durations[i - 1];
+          err = WebPMuxPushFrame(new_mux, &frame, 1);
+          if (err != WEBP_MUX_OK) {
+            ERROR_GOTO2("ERROR: error push frame data #%d\n", i, Err3);
+          }
+          WebPDataClear(&frame.bitstream);
+        }
+        WebPMuxDelete(mux);
+        ok = WriteWebP(new_mux, config->output_);
+        mux = new_mux;  // transfer for the WebPMuxDelete() call
+        new_mux = NULL;
+
+ Err3:
+        free(durations);
+        WebPMuxDelete(new_mux);
+        if (!ok) goto Err2;
+      }
+      break;
+    }
     case ACTION_STRIP: {
       ok = CreateMux(config->input_, &mux);
       if (!ok) goto Err2;
